import unittest

from anyjson import serialize, deserialize

from ghettoq.simple import Connection, Empty


def create_connection(database):
    return Connection(database)


class TestDatabaseBackend(unittest.TestCase):

    def test_basic(self):
        conn = Connection("database")
        q = conn.Queue("testing")

        self.assertRaises(Empty, q.get)
        q.put(serialize({"name": "George Constanza"}))

        self.assertEquals(deserialize(q.get()),
                {"name": "George Constanza"})

    def test_empty_raises_Empty(self):
        conn = create_connection("database")
        q = conn.Queue("testing")

        self.assertRaises(Empty, q.get)

    def test_queue_is_empty_after_purge(self):
        conn = create_connection("database")
        q = conn.Queue("test_queue")
        q.put(serialize({"name": "George Constanza"}))
        q.put(serialize({"name": "George Constanza"}))
        q.purge()

        self.assertRaises(Empty, q.get)

    def test_put__get(self):
        conn = create_connection("database")
        q = conn.Queue("testing")
        q.put(serialize({"name": "George Constanza"}))

        self.assertEquals(deserialize(q.get()),
                {"name": "George Constanza"})

    def test_empty_queueset_raises_Empty(self):
        conn = create_connection("database")
        a, b, c, = conn.Queue("a"), conn.Queue("b"), conn.Queue("c")
        queueset = conn.QueueSet(queue.name for queue in (a, b, c))
        for queue in a, b, c:
            self.assertRaises(Empty, queue.get)
        self.assertRaises(Empty, queueset.get)



import atexit
import socket
import sys

from itertools import count
from django.utils.datastructures import SortedDict
from time import time, sleep

from anyjson import serialize, deserialize
from carrot.backends.base import BaseBackend, BaseMessage
from carrot.utils import gen_unique_id

from ghettoq.backends import Connection
from ghettoq.messaging import Empty as QueueEmpty


class QualityOfService(object):

    def __init__(self, backend, resource, prefetch_count=None,
            interval=None):
        self.backend = backend
        self.resource = resource
        self.prefetch_count = prefetch_count
        self.interval = interval
        self._delivered = SortedDict()
        self._restored_once = False
        atexit.register(self.restore_unacked_once)

    def can_consume(self):
        return len(self._delivered) > self.prefetch_count

    def append(self, message, queue_name, delivery_tag):
        self._delivered[delivery_tag] = message, queue_name

    def ack(self, delivery_tag):
        self._delivered.pop(delivery_tag, None)

    def restore_unacked(self):
        for delivery_tag, delivered in self._delivered.items():
            message, queue_name = delivered
            send = self.backend.prepare_message(message.body, 2, 10,
                                                message.content_type,
                                                message.content_encoding)
            send["destination"] = queue_name
            self.resource.put(queue_name, serialize(send))
        self._delivered = SortedDict()

    def requeue(self, delivery_tag):
        try:
            message, queue_name = self._delivered.pop(delivery_tag)
        except KeyError:
            pass
        self.resource.put(queue_name, message)

    def restore_unacked_once(self):
        if not self._restored_once:
            if self._delivered:
                sys.stderr.write(
                    "Restoring unacknowledged messages: %s\n" % (
                        self._delivered))
            self.restore_unacked()
            if self._delivered:
                sys.stderr.write("UNRESTORED MESSAGES: %s\n" % (
                    self._delivered))


class Message(BaseMessage):

    def __init__(self, backend, payload, **kwargs):
        self.backend = backend

        payload = deserialize(payload)
        kwargs["body"] = payload.get("body").encode("utf-8")
        kwargs["delivery_tag"] = payload.get("delivery_tag")
        kwargs["content_type"] = payload.get("content-type")
        kwargs["content_encoding"] = payload.get("content-encoding")
        kwargs["priority"] = payload.get("priority")
        self.destination = payload.get("destination")

        super(Message, self).__init__(backend, **kwargs)

    def reject(self):
        raise NotImplementedError(
            "The GhettoQ backend does not implement basic.reject")


class MultiBackend(BaseBackend):
    Message = Message
    default_port = None
    type = None
    interval = 1
    polling = True
    _prefetch_count = None
    _consumers = {}
    _callbacks = {}

    def __init__(self, connection, **kwargs):
        if not self.type:
            raise NotImplementedError(
                        "MultiBackends must have the type attribute")
        self.connection = connection
        self._channel = None
        self._qos_manager = None

    def establish_connection(self):
        conninfo = self.connection
        conn = Connection(self.type, host=conninfo.hostname,
                                     user=conninfo.userid,
                                     password=conninfo.password,
                                     database=conninfo.virtual_host,
                                     port=conninfo.port)
        conn.drain_events = self.drain_events
        return conn

    def close_connection(self, connection):
        connection.close()

    def queue_exists(self, queue):
        return True

    def queue_purge(self, queue, **kwargs):
        return self.channel.Queue(queue).purge()

    def _poll(self, resource, timeout=None):
        time_elapsed = 0.0
        while True:
            time_start = time()
            if self.qos_manager.can_consume():
                try:
                    return resource.get()
                except QueueEmpty:
                    pass
            time_elapsed += (time() - time_start)
            if timeout and time_elapsed >= timeout:
                raise socket.timeout("Operation timed out.")
            if self.polling:
                sleep(self.interval)
                time_elapsed += self.interval

    def declare_consumer(self, queue, no_ack, callback, consumer_tag,
                         **kwargs):
        self._consumers[consumer_tag] = queue
        self._callbacks[queue] = callback

    def drain_events(self, timeout=None):
        queueset = self.channel.QueueSet(self._consumers.values())
        payload, queue = self._poll(queueset, timeout=timeout)

        if not queue or queue not in self._callbacks:
            return

        self._callbacks[queue](payload)

    def consume(self, limit=None):
        for total_message_count in count():
            if limit and total_message_count >= limit:
                raise StopIteration

            self.drain_events()

            yield True

    def queue_declare(self, queue, *args, **kwargs):
        pass

    def get(self, queue, **kwargs):
        try:
            payload = self.channel.Queue(queue).get()
        except QueueEmpty:
            return None
        else:
            return self.message_to_python(payload)

    def ack(self, delivery_tag):
        self.qos_manager.ack(delivery_tag)

    def requeue(self, delivery_tag):
        self.qos_manager.requeue(delivery_tag)

    def message_to_python(self, raw_message):
        message = self.Message(backend=self, payload=raw_message)
        self.qos_manager.append(message, message.destination,
                                message.delivery_tag)
        return message

    def prepare_message(self, message_data, delivery_mode, priority=0,
            content_type=None, content_encoding=None):
        return {"body": message_data,
                "delivery_tag": gen_unique_id(),
                "priority": priority or 0,
                "content-encoding": content_encoding,
                "content-type": content_type}

    def publish(self, message, exchange, routing_key, **kwargs):
        message["destination"] = exchange
        self.channel.Queue(exchange).put(serialize(message),
                                         priority=message["priority"])

    def cancel(self, consumer_tag):
        if not self._channel:
            return
        queue = self._consumers.pop(consumer_tag, None)
        self._callbacks.pop(queue, None)

    def close(self):
        for consumer_tag in self._consumers.keys():
            self.cancel(consumer_tag)
        if self._channel:
            self._channel.close()
        self._channel = None

    def basic_qos(self, prefetch_size, prefetch_count, apply_global=False):
        self._prefetch_count = prefetch_count

    @property
    def channel(self):
        if not self._channel:
            # Need one connection per channel.
            # AMQP has multiplexing, but Redis does not.
            self._channel = self.establish_connection()
        return self._channel

    @property
    def qos_manager(self):
        if self._qos_manager is None:
            self._qos_manager = QualityOfService(self, self.channel)

        # Update prefetch count / interval
        self._qos_manager.prefetch_count = self._prefetch_count
        self._qos_manager.interval = self.interval

        return self._qos_manager


class Redis(MultiBackend):
    type = "Redis"
    polling = False


class Database(MultiBackend):
    type = "database"


class MongoDB(MultiBackend):
    type = "mongodb"


class Beanstalk(MultiBackend):
    type = "Beanstalk"
    polling = False



# Partially stolen from Django Queue Service
# (http://code.google.com/p/django-queue-service)
from django.db import models


class QueueManager(models.Manager):

    def publish(self, queue_name, payload):
        queue, created = self.get_or_create(name=queue_name)
        queue.messages.create(payload=payload)

    def fetch(self, queue_name):
        try:
            queue = self.get(name=queue_name)
        except self.model.DoesNotExist:
            return

        return queue.messages.pop()

    def purge(self, queue_name):
        try:
            queue = self.get(name=queue_name)
        except self.model.DoesNotExist:
            return

        messages = queue.messages.all()
        count = messages.count()
        messages.delete()
        return count


class MessageManager(models.Manager):

    def pop(self):
        try:
            resultset = self.filter(visible=True).order_by('sent_at', 'id')
            result = resultset[0:1].get()
            result.visible = False
            result.save()
            return result.payload
        except self.model.DoesNotExist:
            pass

    def cleanup(self):
        self.filter(visible=False).delete()



from ghettoq import messaging


class BaseBackend(object):

    def __init__(self, host=None, port=None, user=None, password=None,
            database=None, timeout=None):
        self.host = host
        self.port = port
        self.user = user
        self.password = password
        self.database = database
        self.timeout = timeout
        self.connection = None

    def Queue(self, name):
        return messaging.Queue(self, name)

    def QueueSet(self, names):
        return messaging.QueueSet(self, names)

    @property
    def client(self):
        if self.connection is None:
            self.connection = self.establish_connection()
        return self.connection

    def close(self):
        pass



import sys

BACKEND_ALIASES = {
    "redis": "ghettoq.backends.pyredis.RedisBackend",
    "database": "ghettoq.backends.database.DatabaseBackend",
    "mongodb": "ghettoq.backends.mongodb.MongodbBackend",
    "beanstalk": "ghettoq.backends.beanstalk.BeanstalkBackend",
}
_backend_cache = {}


def resolve_backend(backend):
    backend = BACKEND_ALIASES.get(backend.lower(), backend)
    backend_module_name, _, backend_cls_name = backend.rpartition(".")
    return backend_module_name, backend_cls_name


def _get_backend_cls(backend):
    backend_module_name, backend_cls_name = resolve_backend(backend)

    __import__(backend_module_name)
    backend_module = sys.modules[backend_module_name]
    return getattr(backend_module, backend_cls_name)


def get_backend_cls(backend):
    if backend not in _backend_cache:
        _backend_cache[backend] = _get_backend_cls(backend)
    return _backend_cache[backend]


def Connection(type, *args, **kwargs):
    return get_backend_cls(type)(*args, **kwargs)



from pymongo.connection import Connection
from pymongo.errors import OperationFailure

from ghettoq.backends.base import BaseBackend
from ghettoq.messaging import Empty

__author__ = "Flavio [FlaPer87] Percoco Premoli <flaper87@flaper87.org>"

DEFAULT_HOST = "127.0.0.1"
DEFAULT_PORT = 27017


class MongodbBackend(BaseBackend):

    def establish_connection(self):
        self.port = self.port or DEFAULT_PORT
        self.host = self.host or DEFAULT_HOST
        self.connection = Connection(host=self.host, port=self.port)
        dbname = self.database
        if not dbname or dbname == "/":
            dbname = "ghettoq"
        self.database = getattr(self.connection, dbname)
        col = self.database.messages
        col.ensure_index([("queue", 1)])
        return col

    def put(self, queue, message, **kwargs):
        self.client.insert({"payload": message, "queue": queue})

    def get(self, queue):
        try:
            msg = self.client.database.command("findandmodify",
                        "messages", query={"queue": queue}, remove=True)
        except OperationFailure:
            raise Empty("Empty queue")
        return msg["value"]["payload"]

    def purge(self, queue):
        return self.client.remove({"queue": queue})



from Queue import Empty
from itertools import cycle


class Queue(object):

    def __init__(self, backend, name):
        self.name = name
        self.backend = backend

    def put(self, payload, priority=0):
        self.backend.put(self.name, payload, priority=priority)

    def get(self):
        payload = self.backend.get(self.name)
        if payload is not None:
            return payload
        raise Empty

    def purge(self):
        self.backend.purge(self.name)

    def __repr__(self):
        return "<Queue: %s>" % repr(self.name)


class QueueSet(object):
    """A set of queues that operates as one."""

    def __init__(self, backend, queues):
        self.backend = backend
        self.queue_names = list(queues)

        self._get_many = getattr(self.backend, "get_many", None)
        self.get = self._emulated
        if self._get_many:
            self.get = self._native

        # attributes below are only used in emulation mode.

        # queues could be a PriorityQueue as well to support
        # priorities.
        self.queues = map(self.backend.Queue, self.queue_names)

        # an infinite cycle through all the queues.
        self.cycle = cycle(self.queues)

        # A set of all the queue names, so we can match when we've
        # tried all of them.
        self.all = frozenset(self.queue_names)

    def _native(self, timeout=None):
        return self._get_many(self.queue_names, timeout=timeout)

    def _emulated(self, timeout=None):
        """Get the next message avaiable in the queue.

        :returns: The message and the name of the queue it came from as
            a tuple.
        :raises Empty: If there are no more items in any of the queues.

        """

        # A set of queues we've already tried.
        tried = set()

        while True:
            # Get the next queue in the cycle, and try to get an item off it.
            try:
                queue = self.cycle.next()
            except StopIteration:
                raise Empty("No queues registered")
            try:
                item = queue.get()
            except Empty:
                # raises Empty when we've tried all of them.
                tried.add(queue.name)
                if tried == self.all:
                    raise
            else:
                return item, queue.name

    def __repr__(self):
        return "<QueueSet: %s>" % repr(self.queue_names)



from ghettoq.models import Queue
from ghettoq.backends.base import BaseBackend


class DatabaseBackend(BaseBackend):

    def __init__(self, *args, **kwargs):
        from django.conf import settings
        if not settings.configured:
            settings.configure(DEBUG=False,
                               DATABASE_HOST=self.host,
                               DATABASE_PORT=self.port,
                               DATABASE_NAME=self.database,
                               DATABASE_USER=self.user,
                               DATABASE_PASSWORD=self.password)
        super(DatabaseBackend, self).__init__(*args, **kwargs)

    def establish_connection(self):
        pass

    def put(self, queue, message, **kwargs):
        Queue.objects.publish(queue, message)

    def get(self, queue):
        self.refresh_connection()
        return Queue.objects.fetch(queue)

    def purge(self, queue):
        return Queue.objects.purge(queue)

    def refresh_connection(self):
        from django.db import connection
        connection.close()



from django.conf import settings
from django.test.simple import run_tests as django_test_runner


def run_tests(test_labels, verbosity=1, interactive=True, extra_tests=None,
        **kwargs):
    """ Test runner that only runs tests for the apps
    listed in ``settings.TEST_APPS``.
    """
    extra_tests = extra_tests or []
    app_labels = getattr(settings, "TEST_APPS", test_labels)
    return django_test_runner(app_labels,
                              verbosity=verbosity, interactive=interactive,
                              extra_tests=extra_tests, **kwargs)



from itertools import ifilter
from Queue import Empty

from beanstalkc import Connection
from ghettoq.backends.base import BaseBackend

DEFAULT_HOST = 'localhost'
DEFAULT_PORT = 11300


class BeanstalkBackend(BaseBackend):

    def _parse_job(self, job):
        item, dest = None, None
        if job:
            try:
                item = job.body
                dest = job.stats()['tube']
            except:
                job.bury()
            else:
                job.delete()
        else:
            raise Empty
        return item, dest

    def establish_connection(self):
        self.host = self.host or DEFAULT_HOST
        self.port = self.port or DEFAULT_PORT
        return Connection(host=self.host, port=self.port)

    def put(self, queue, message, priority=0, **kwargs):
        self.client.use(queue)
        self.client.put(message, priority=priority)

    def get(self, queue):
        if not queue:
            raise Empty
        if queue not in self.client.watching():
            self.client.watch(queue)

        ignore = ifilter(lambda q: q!= queue, self.client.watching())
        map(self.client.ignore, ignore)

        job = self.client.reserve(timeout=1)
        item, dest = self._parse_job(job)
        return item

    def get_many(self, queues, timeout=None):
        if not queues:
            raise Empty

        # timeout of None will cause beanstalk to timeout waiting
        # for a new request
        if timeout is None:
            timeout = 1

        to_watch = ifilter(lambda q: q not in self.client.watching(), queues)
        map(self.client.watch, to_watch)

        job = self.client.reserve(timeout=timeout)
        return self._parse_job(job)

    def purge(self, queue):
        raise NotImplementedError



import unittest

from anyjson import serialize, deserialize

from ghettoq.simple import Connection, Empty


def create_connection(database):
    return Connection("redis", host="localhost", database=database)


class TestRedisBackend(unittest.TestCase):

    def test_default_database_is_set_correctly(self):
        conn1 = create_connection("")
        conn2 = create_connection("/")
        conn3 = create_connection("")
        conn4 = create_connection(None)

        self.assertEquals(conn1.database, 0)
        self.assertEquals(conn2.database, 0)
        self.assertEquals(conn3.database, 0)
        self.assertEquals(conn4.database, 0)

    def test_database_name_is_set_correctly(self):
        conn1 = create_connection("1")
        conn2 = create_connection("/2")
        conn3 = create_connection(3)

        self.assertEquals(conn1.database, 1)
        self.assertEquals(conn2.database, 2)
        self.assertEquals(conn3.database, 3)

    def test_invalid_database_name_raises_AttributeError(self):
        self.assertRaises(AttributeError, create_connection, "string")
        self.assertRaises(AttributeError, create_connection, "1string")
        self.assertRaises(AttributeError, create_connection, "/string")

    def test_empty_raises_Empty(self):
        conn = create_connection(1)
        q = conn.Queue("testing")

        self.assertRaises(Empty, q.get)

    def test_queue_is_empty_after_purge(self):
        conn = create_connection(1)
        q = conn.Queue("test_queue")
        q.put(serialize({"name": "George Constanza"}))
        q.put(serialize({"name": "George Constanza"}))
        q.purge()

        self.assertRaises(Empty, q.get)

    def test_put__get(self):
        conn = create_connection(1)
        q = conn.Queue("testing")
        q.put(serialize({"name": "George Constanza"}))

        self.assertEquals(deserialize(q.get()),
                {"name": "George Constanza"})

    def test_empty_queueset_raises_Empty(self):
        conn = create_connection(1)
        a, b, c, = conn.Queue("a"), conn.Queue("b"), conn.Queue("c")
        queueset = conn.QueueSet(queue.name for queue in (a, b, c))
        for queue in a, b, c:
            self.assertRaises(Empty, queue.get)
        self.assertRaises(Empty, queueset.get)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import codecs

try:
    from setuptools import setup, find_packages, Command
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages, Command

from distutils.command.install_data import install_data
from distutils.command.install import INSTALL_SCHEMES
import sys

import ghettoq

packages, data_files = [], []
root_dir = os.path.dirname(__file__)
if root_dir != '':
    os.chdir(root_dir)
src_dir = "ghettoq"


def osx_install_data(install_data):

    def finalize_options(self):
        self.set_undefined_options("install", ("install_lib", "install_dir"))
        install_data.finalize_options(self)


class RunTests(Command):
    description = "Run the django test suite from the testproj dir."

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        this_dir = os.getcwd()
        testproj_dir = os.path.join(this_dir, "testproj")
        os.chdir(testproj_dir)
        sys.path.insert(0, testproj_dir)
        from django.core.management import execute_manager
        os.environ["DJANGO_SETTINGS_MODULE"] = os.environ.get(
                        "DJANGO_SETTINGS_MODULE", "settings")
        settings_file = os.environ["DJANGO_SETTINGS_MODULE"]
        settings_mod = __import__(settings_file, {}, {}, [''])
        execute_manager(settings_mod, argv=[
            __file__, "test"])
        os.chdir(this_dir)


def fullsplit(path, result=None):
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)


for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']

SKIP_EXTENSIONS = [".pyc", ".pyo", ".swp", ".swo"]


def is_unwanted_file(filename):
    for skip_ext in SKIP_EXTENSIONS:
        if filename.endswith(skip_ext):
            return True
    return False


for dirpath, dirnames, filenames in os.walk(src_dir):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith("."):
            del dirnames[i]
    for filename in filenames:
        if filename.endswith(".py"):
            packages.append('.'.join(fullsplit(dirpath)))
        elif is_unwanted_file(filename):
            pass
        else:
            data_files.append([dirpath, [os.path.join(dirpath, f) for f in
                filenames]])

setup(
    name='ghettoq',
    version=ghettoq.__version__,
    description=ghettoq.__doc__,
    author=ghettoq.__author__,
    author_email=ghettoq.__contact__,
    url=ghettoq.__homepage__,
    platforms=["any"],
    license='BSD',
    packages=packages,
    data_files=data_files,
    cmdclass = {"test": RunTests},
    zip_safe=False,
    test_suite="nose.collector",
    install_requires=[
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    long_description=codecs.open('README', "r", "utf-8").read(),
)



"""Ghetto Queue using Redis or Django Models."""
VERSION = (0, 4, 4)
__version__ = ".".join(map(str, VERSION))
__author__ = "Ask Solem"
__contact__ = "askh@opera.com"
__homepage__ = "http://github.com/ask/ghettoq/"
__docformat__ = "restructuredtext"
__license__ = "BSD"



from Queue import Empty

from redis import Redis
from ghettoq.backends.base import BaseBackend

DEFAULT_PORT = 6379
DEFAULT_DB = 0


class RedisBackend(BaseBackend):

    def __init__(self, host=None, port=None, user=None, password=None,
            database=None, timeout=None):

        if not isinstance(database, int):
            if not database or database == "/":
                database = DEFAULT_DB
            elif database.startswith('/'):
                database = database[1:]
            try:
                database = int(database)
            except ValueError:
                raise AttributeError(
                    "Database name must be integer between 0 "
                    "and database_count - 1")

        super(RedisBackend, self).__init__(host, port, user, password,
                                           database, timeout)

    def establish_connection(self):
        self.port = self.port or DEFAULT_PORT
        return Redis(host=self.host, port=self.port, db=self.database,
                     password=self.password)

    def put(self, queue, message, **kwargs):
        self.client.lpush(queue, message)

    def get(self, queue):
        if not queue:
            raise Empty

        try:
            dest, item = self.client.brpop([queue], timeout=1)
        except TypeError:
            raise Empty

        return item

    def get_many(self, queues, timeout=None):
        if not queues:
            raise Empty

        try:
            dest, item = self.client.brpop(queues, timeout=1)
        except TypeError:
            raise Empty

        return item, dest

    def purge(self, queue):
        size = self.client.llen(queue)
        self.client.delete(queue)
        return size
